// src/index.ts
import { CodegenContext, generate, loadContext } from "@graphql-codegen/cli";

// src/utils/isCodegenConfig.ts
import { normalizePath } from "vite";
function isCodegenConfig(filePath, context) {
  return normalizePath(filePath) === normalizePath(context.filepath);
}

// src/utils/isGraphQLFile.ts
import { normalizePath as normalizePath2 } from "vite";
import { normalizeInstanceOrArray } from "@graphql-codegen/plugin-helpers";
async function isGraphQLFile(filePath, context) {
  const { documents = [], schema = [] } = context.getConfig();
  const normalized = [
    ...normalizeInstanceOrArray(documents),
    ...normalizeInstanceOrArray(schema)
  ];
  const loadedDocuments = await context.loadDocuments(normalized);
  if (!loadedDocuments.length)
    return false;
  const paths = loadedDocuments.map(({ location = "" }) => location).map(normalizePath2).filter(Boolean);
  if (!paths.length)
    return false;
  return paths.some((documentPath) => documentPath === normalizePath2(filePath));
}

// src/utils/viteModes.ts
var modes = {
  serve: "serve",
  build: "build"
};
var isServeMode = (mode) => mode === modes.serve;
var isBuildMode = (mode) => mode === modes.build;

// src/utils/debugLog.ts
var RESET = "\x1B[0m";
var BRIGHT = "\x1B[1m";
var DIM = "\x1B[2m";
var FG_CYAN = "\x1B[36m";
function debugLog(...args) {
  const LOG_PREFIX = `${FG_CYAN}${BRIGHT}VITE PLUGIN GRAPHQL CODEGEN${RESET} `;
  console.log(LOG_PREFIX, DIM, ...args, RESET);
}

// src/index.ts
function VitePluginGraphQLCodegen(options) {
  let codegenContext;
  let viteMode;
  const {
    runOnStart = true,
    runOnBuild = true,
    enableWatcher = true,
    config = null,
    configOverride = {},
    configOverrideOnStart = {},
    configOverrideOnBuild = {},
    configOverrideWatcher = {},
    configFilePathOverride,
    debug = false
  } = options ?? {};
  const log = (...args) => {
    if (!debug)
      return;
    debugLog(...args);
  };
  const generateWithOverride = async (overrideConfig) => {
    const currentConfig = codegenContext.getConfig();
    return generate({
      ...currentConfig,
      ...configOverride,
      ...overrideConfig,
      watch: false
    });
  };
  log("Plugin initialized with options:", options);
  return {
    name: "graphql-codegen",
    async config(_config, env) {
      try {
        if (config) {
          log("Manual config passed, creating codegen context");
          codegenContext = new CodegenContext({ config });
        } else {
          const cwd = process.cwd();
          log("Loading codegen context:", configFilePathOverride ?? cwd);
          codegenContext = await loadContext(configFilePathOverride);
        }
        log("Loading codegen context successful");
      } catch (error) {
        log("Loading codegen context failed");
        throw error;
      }
      viteMode = env.command;
    },
    async buildStart() {
      const isServe = isServeMode(viteMode);
      const isBuild = isBuildMode(viteMode);
      if (isServe && !runOnStart)
        return;
      if (isBuild && !runOnBuild)
        return;
      try {
        if (isServe)
          await generateWithOverride(configOverrideOnStart);
        if (isBuild)
          await generateWithOverride(configOverrideOnBuild);
        isServe && log("Generation successful on start");
        isBuild && log("Generation successful on build");
      } catch (error) {
        isServe && log("Generation failed on start");
        isBuild && log("Generation failed on build");
        if (isBuild)
          throw error;
      }
    },
    configureServer(server) {
      const listener = async (filePath = "") => {
        log("File changed:", filePath);
        const isConfig = isCodegenConfig(filePath, codegenContext);
        if (isConfig) {
          log("Codegen config file changed, restarting vite");
          server.restart();
          return;
        }
        if (!enableWatcher)
          return;
        try {
          const isDocument = await isGraphQLFile(filePath, codegenContext);
          log("Document check successful in file watcher");
          if (!isDocument)
            return;
        } catch (error) {
          log("Document check failed in file watcher");
        }
        log("File matched a graphql document");
        try {
          await generateWithOverride(configOverrideWatcher);
          log("Generation successful in file watcher");
        } catch (error) {
          log("Generation failed in file watcher");
        }
      };
      server.watcher.on("add", listener);
      server.watcher.on("change", listener);
    }
  };
}
export {
  VitePluginGraphQLCodegen as default
};
//# sourceMappingURL=index.js.map